@@ -1,9 +1,9 @@
 import asyncio
 import signal
 import sys
-from asyncio import gather, get_event_loop, iscoroutine
+from asyncio import gather, get_event_loop
 from asyncio.events import AbstractEventLoop
-from typing import Any, Awaitable, Callable, List, Optional, Set, Union, cast
+from typing import Any, Callable, List, Optional, Set, Union
 
 from aiocli.commander_app import (
     Application,
@@ -44,7 +44,7 @@ def _cancel_tasks(to_cancel: Set['asyncio.Task[Any]'], loop: AbstractEventLoop)
         return
     for task in to_cancel:
         task.cancel()
-    loop.run_until_complete(gather(*to_cancel, loop=loop, return_exceptions=True))
+    loop.run_until_complete(gather(*to_cancel, return_exceptions=True))
     for task in to_cancel:
         if task.cancelled():
             continue
@@ -78,7 +78,7 @@ class AppRunner:
     def app(self) -> Application:
         return self._app
 
-    async def setup(self) -> None:
+    async def setup(self, all_hooks: bool = True) -> None:
         if self._handle_signals:
             try:
                 self._loop.add_signal_handler(signal.SIGINT, _raise_graceful_exit)
@@ -86,10 +86,10 @@ class AppRunner:
             except NotImplementedError:  # pragma: no cover
                 # add_signal_handler is not implemented on Windows
                 pass
-        await self.startup()
+        await self.startup(all_hooks=all_hooks)
 
-    async def startup(self) -> None:
-        await self._app.startup()
+    async def startup(self, all_hooks: bool) -> None:
+        await self._app.startup(all_hooks=all_hooks)
 
     async def shutdown(self) -> None:
         await self._app.shutdown()
@@ -109,20 +109,28 @@ class AppRunner:
 
 
 async def _run_app(
-    app: Union[Application, Awaitable[Application]],
+    app: Application,
     *,
     loop: AbstractEventLoop,
     handle_signals: bool = True,
     argv: Optional[List[str]] = None,
     exit_code: bool = True,
 ) -> None:
-    app = cast(Application, await app if iscoroutine(app) else app)  # type: ignore
     runner = AppRunner(app, loop=loop, handle_signals=handle_signals, exit_code=exit_code)
-    await runner.setup()
-    try:
-        await app(argv or sys.argv[1:])
-    finally:
-        await runner.cleanup()
+    args = argv or sys.argv[1:]
+    if app.should_ignore_hooks(args):
+        await runner.setup(all_hooks=False)
+        try:
+            await app(args)
+        finally:
+            if exit_code:
+                app.exit()
+    else:
+        await runner.setup(all_hooks=True)
+        try:
+            await app(args)
+        finally:
+            await runner.cleanup()
 
 
 ApplicationParser = Callable[..., Optional[List[str]]]
@@ -131,7 +139,7 @@ ApplicationReturn = Union[int, None, Callable[..., Optional[int]]]
 
 
 def run_app(
-    app: Union[Application, Awaitable[Application]],
+    app: Application,
     *,
     loop: Optional[AbstractEventLoop] = None,
     handle_signals: bool = True,
@@ -161,6 +169,6 @@ def run_app(
                 loop_.run_until_complete(loop_.shutdown_asyncgens())
             if close_loop and not loop_.is_closed():
                 loop_.close()
-        return cast(Application, app).exit_code
+        return app.exit_code
 
     return wrapper() if parser is None else wrapper
