@@ -1,4 +1,6 @@
+from abc import ABC, abstractmethod
 from argparse import Action, ArgumentParser, RawTextHelpFormatter
+from dataclasses import dataclass, field
 from inspect import signature
 from typing import (
     Any,
@@ -14,6 +16,7 @@ from typing import (
     Tuple,
     Type,
     Union,
+    cast,
 )
 
 __all__ = (
@@ -33,7 +36,7 @@ from .logger import logger
 CommandHandler = Callable[..., Union[Callable[[Any], Optional[int]], Coroutine[Any, Any, Optional[int]]]]
 
 
-class State(dict):
+class State(Dict[Any, Any]):
     pass
 
 
@@ -63,7 +66,7 @@ class CommandArgument(NamedTuple):
     dest: Optional[str] = None
 
 
-# dataclass
+@dataclass
 class Command:
     name: str
     handler: CommandHandler
@@ -72,44 +75,17 @@ class Command:
             Tuple[str, Dict[str, Any]],
             CommandArgument,
         ]
-    ]
+    ] = field(default_factory=list)
     optionals: List[
         Union[
             Tuple[str, Dict[str, Any]],
             CommandArgument,
         ]
-    ]
-    deprecated: Optional[bool]
-    description: Optional[str]
-    usage: Optional[str]
-
-    def __init__(
-        self,
-        name: str,
-        handler: CommandHandler,
-        positionals: List[
-            Union[
-                Tuple[str, Dict[str, Any]],
-                CommandArgument,
-            ]
-        ],
-        optionals: List[
-            Union[
-                Tuple[str, Dict[str, Any]],
-                CommandArgument,
-            ]
-        ],
-        deprecated: Optional[bool] = None,
-        description: Optional[str] = None,
-        usage: Optional[str] = None,
-    ) -> None:
-        self.name = name
-        self.handler = handler  # type: ignore
-        self.positionals = positionals
-        self.optionals = optionals
-        self.deprecated = deprecated
-        self.description = description
-        self.usage = usage
+    ] = field(default_factory=list)
+    deprecated: Optional[bool] = None
+    description: Optional[str] = None
+    usage: Optional[str] = None
+    ignore_hooks: bool = False
 
 
 def command(
@@ -161,6 +137,12 @@ ArgumentState = Union[
 ]
 
 
+class InternalCommandHook(ABC):
+    @abstractmethod
+    async def __call__(self, app: 'Application') -> None:
+        pass
+
+
 class Application:
     _parser: ArgumentParser
     _parsers: Dict[str, ArgumentParser]
@@ -215,7 +197,7 @@ class Application:
                 _ = self._parser.parse_args([name])
                 return default_exit_code
 
-            return Command(name=name, handler=self_handler, positionals=[], optionals=[], deprecated=False)
+            return Command(name=name, handler=self_handler, deprecated=False, ignore_hooks=True)
 
         self._commands = {
             '-h': self_command('-h'),
@@ -229,10 +211,14 @@ class Application:
         self._middleware = [] if middleware is None else list(middleware)
         self._exception_handlers = {} if exception_handlers is None else exception_handlers
 
-        async def self_startup(app_: 'Application') -> None:
-            app_.set_state(state=state or State())
+        class SelfStartupInternalCommandHook(InternalCommandHook):
+            async def __call__(self, app: 'Application') -> None:
+                app.set_state(state=state or State())
 
-        self._on_startup = [self_startup, *([] if on_startup is None else list(on_startup))]
+        self._on_startup = [
+            SelfStartupInternalCommandHook().__call__,
+            *([] if on_startup is None else list(on_startup)),
+        ]
         self._on_shutdown = [] if on_shutdown is None else list(on_shutdown)
         self._on_cleanup = [] if on_cleanup is None else list(on_cleanup)
         self._deprecated = bool(deprecated)
@@ -270,11 +256,12 @@ class Application:
 
     def include_router(self, router: 'Application') -> None:
         for name, cmd in router._commands.items():
-            if cmd.deprecated is None:
-                cmd.deprecated = self._deprecated
-            self._commands[name] = cmd
-            self._parsers[name] = router._parsers[name]
-            self._update_parser_description(name)
+            if name not in self._commands:
+                if cmd.deprecated is None:
+                    cmd.deprecated = self._deprecated
+                self._commands[name] = cmd
+                self._parsers[name] = router._parsers[name]
+                self._update_parser_description(name)
         for middleware in router._middleware:
             self._middleware.append(middleware)
         self._exception_handlers.update(router._exception_handlers)
@@ -289,28 +276,52 @@ class Application:
         self,
         name: str,
         *,
-        positionals: Optional[List[Tuple[str, Dict[str, Any]]]] = None,
-        optionals: Optional[List[Tuple[str, Dict[str, Any]]]] = None,
+        positionals: Optional[
+            List[
+                Union[
+                    Tuple[str, Dict[str, Any]],
+                    CommandArgument,
+                ]
+            ]
+        ] = None,
+        optionals: Optional[
+            List[
+                Union[
+                    Tuple[str, Dict[str, Any]],
+                    CommandArgument,
+                ]
+            ]
+        ] = None,
         deprecated: Optional[bool] = None,
         description: Optional[str] = None,
         usage: Optional[str] = None,
+        ignore_hooks: bool = False,
     ) -> Callable[[CommandHandler], CommandHandler]:
         def decorator(handler: CommandHandler) -> CommandHandler:
             self._add_command(
                 Command(
                     name=name,
                     handler=handler,
-                    positionals=positionals or [],  # type: ignore
-                    optionals=optionals or [],  # type: ignore
+                    positionals=positionals or [],
+                    optionals=optionals or [],
                     deprecated=deprecated,
                     description=description,
                     usage=usage,
+                    ignore_hooks=ignore_hooks,
                 )
             )
             return handler
 
         return decorator
 
+    def should_ignore_hooks(self, args: List[str]) -> bool:
+        cmd: Optional[Command] = None
+        if len(args) > 0:
+            cmd = self.get_command(name=args[0])
+        if not cmd:
+            cmd = self.get_command(name=self._default_command)
+        return not cmd or cmd.ignore_hooks
+
     def add_commands(self, commands: Sequence[Command]) -> None:
         for cmd in commands:
             self._add_command(cmd)
@@ -367,8 +378,8 @@ class Application:
     def on_startup(self) -> List[CommandHook]:
         return self._on_startup
 
-    async def startup(self) -> None:
-        await self._execute_command_hooks(self.on_startup)
+    async def startup(self, all_hooks: bool = True) -> None:
+        await self._execute_command_hooks(self.on_startup, all_hooks)
 
     @property
     def on_shutdown(self) -> List[CommandHook]:
@@ -423,8 +434,11 @@ class Application:
                 *([handler] if len(signature(handler).parameters) == 0 else [handler, cmd, kwargs])  # type: ignore
             )
 
-    async def _execute_command_hooks(self, command_hooks: List[CommandHook]) -> None:
-        for hook in command_hooks:
+    async def _execute_command_hooks(self, command_hooks: List[CommandHook], all_hooks: bool = True) -> None:
+        command_hooks_ = (
+            command_hooks if all_hooks else [hook for hook in command_hooks if isinstance(hook, InternalCommandHook)]
+        )
+        for hook in command_hooks_:
             self._log(
                 msg='Executing hook "{0}" ({1})'.format(
                     hook.__name__ if hasattr(hook, '__name__') else 'unknown', id(hook)
@@ -510,7 +524,7 @@ class Application:
                 ', '.join(['{0}={1}'.format(key, val) for key, val in kwargs.items()])
             ),
         )
-        return await resolve_function(handler, **kwargs)
+        return cast(int, await resolve_function(handler, **kwargs))
 
     async def _execute_command_exception_handler(
         self,
@@ -533,7 +547,7 @@ class Application:
                 ', '.join(['{0}={1}'.format(key, val) for key, val in kwargs.items()]),
             )
         )
-        return await resolve_function(exception_handler, err, cmd, kwargs)
+        return cast(Optional[int], await resolve_function(exception_handler, err, cmd, kwargs))
 
     def set_state(self, state: ArgumentState) -> None:
         if isinstance(state, State):
@@ -549,5 +563,5 @@ class Application:
 
     def _log(self, msg: str) -> None:
         if self._debug:
-            print(msg)
+            # print(msg)
             logger.debug(msg)
