@@ -26,7 +26,7 @@ class TestCommander:
         timeout_exit_code: Optional[int] = None,
     ) -> int:
         try:
-            await wait_for(self._app.__call__(argv or sys.argv[1:]), timeout=timeout)
+            await wait_for(self._app(argv or sys.argv[1:]), timeout=timeout)
         except TimeoutError:
             if timeout_exit_code is not None:
                 return timeout_exit_code
@@ -36,7 +36,7 @@ class TestCommander:
         if self._runner:
             return
         self._runner = await self._make_runner(**kwargs)
-        await self._runner.setup()
+        await self._runner.setup(all_hooks=True)
 
     async def __aenter__(self) -> 'TestCommander':
         await self.start_commander()
